from celery import Celery


celery_app = Celery()
celery_app.conf.CELERY_ALWAYS_EAGER = True


class DB(dict):
    def __init__(self, *a, **kw):
        super(DB, self).__init__(*a, **kw)
        self.reset()

    def reset(self):
        self['rows'] = []


db = DB()


@celery_app.task()
def dummytask1(*args, **kwargs):
    db['rows'].append('1')


@celery_app.task
def dummytask2(*args, **kwargs):
    db['rows'].append('2')


@celery_app.task
def dummytask_args_kwargs(*args, **kwargs):
    db['rows'].append((args, kwargs))


def dummymethod1(*args, **kwargs):
    dummytask1.s(*args, **kwargs).delay()


def dummymethod2(*args, **kwargs):
    dummytask2.s(*args, **kwargs).delay()


def dummymethod_args_kwargs(*args, **kwargs):
    dummytask_args_kwargs.s(*args, **kwargs).delay()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_celery_mock
----------------------------------

Tests for `celery_mock` module.
"""

import pytest

from celery_mock import task_mock
from celery_mock.celery_mock import AlreadyMockedError, NotStartedError
from .example_app import (
    db,
    dummymethod1,
    dummymethod2,
    dummymethod_args_kwargs,
)


@pytest.yield_fixture(autouse=True)
def reset_db():
    db.reset()
    yield


def test_context_manager():
    dummymethod1()

    assert db['rows'] == ['1']

    with task_mock():
        dummymethod1()
        dummymethod1()
        assert db['rows'] == ['1']

    assert db['rows'] == ['1', '1', '1']

    dummymethod1()

    assert db['rows'] == ['1', '1', '1', '1']


def test_start_stop():
    tmock = task_mock()

    dummymethod1()
    assert db['rows'] == ['1']

    tmock.start()
    dummymethod1()
    dummymethod1()
    assert db['rows'] == ['1']
    tmock.stop()
    assert db['rows'] == ['1', '1', '1']

    dummymethod1()

    assert db['rows'] == ['1', '1', '1', '1']


def test_mock_single_task():
    dummymethod1()
    dummymethod2()

    assert db['rows'] == ['1', '2']

    with task_mock('tests.example_app.dummytask1'):
        dummymethod1()
        dummymethod2()
        assert db['rows'] == ['1', '2', '2']

    assert db['rows'] == ['1', '2', '2', '1']

    dummymethod1()
    dummymethod2()

    assert db['rows'] == ['1', '2', '2', '1', '1', '2']


def test_mock_both_tasks():
    tmock1 = task_mock('tests.example_app.dummytask1').start()
    tmock2 = task_mock('tests.example_app.dummytask2').start()

    dummymethod1()
    dummymethod2()

    assert db['rows'] == []

    tmock2.stop()

    assert db['rows'] == ['2']

    tmock1.stop()

    assert db['rows'] == ['2', '1']


def test_call_arguments_assertion():
    with task_mock('tests.example_app.dummytask1') as tmock1:
        with task_mock('tests.example_app.dummytask2') as tmock2:
            dummymethod2('22', b='bb')
            assert len(tmock1.calls) == 0
            assert len(tmock2.calls) == 1
            task, args, kwargs = tmock2.calls[0]
            assert task.name == 'tests.example_app.dummytask2'
            assert args == ('22',)
            assert kwargs == {'b': 'bb'}
            dummymethod1('11', a='aa')
        assert db['rows'] == ['2']
        assert len(tmock1.calls) == 1
        assert len(tmock2.calls) == 0
        task, args, kwargs = tmock1.calls[0]
        assert task.name == 'tests.example_app.dummytask1'
        assert args == ('11',)
        assert kwargs == {'a': 'aa'}
    assert db['rows'] == ['2', '1']
    assert len(tmock1.calls) == 0
    assert len(tmock2.calls) == 0


def test_call_args():
    with task_mock('tests.example_app.dummytask_args_kwargs'):
        dummymethod_args_kwargs('a1', a='aa1')
        dummymethod_args_kwargs('a2', a='aa2')
        dummymethod_args_kwargs('a3', a='aa3')

    assert db['rows'] == [
        (('a1',), {'a': 'aa1'}),
        (('a2',), {'a': 'aa2'}),
        (('a3',), {'a': 'aa3'}),
    ]


def test_exceptions():
    tmock = task_mock().start()
    with pytest.raises(AlreadyMockedError):
        task_mock().start()

    with pytest.raises(NotStartedError):
        task_mock().stop()
    tmock.stop()
    task_mock().start()

    with pytest.raises(AlreadyMockedError):
        task_mock().start()

    with pytest.raises(AlreadyMockedError):
        task_mock('tests.example_app.dummytask_args_kwargs').start()



# -*- coding: utf-8 -*-
from unittest import mock
from celery import Task, states
from celery.result import EagerResult
from celery.utils import uuid


class ALL(object):
    pass


class AlreadyMockedError(Exception):
    pass


class NotStartedError(Exception):
    pass


_super_apply_async = Task.apply_async
_mocked_tasks = dict()


class CeleryTaskMock(object):
    """
    Mocks celery task so that it won't get run until `runtasks` method call.

    >>> cmock = task_mock('myapp.tasks.dummy_task')
    >>> cmock.start()
    >>> mydummyfunction()  # triggers dummy_task
    >>> cmock.stop()  # runs dummy_task

    or

    >>> with task_mock('myapp.tasks.dummy_task'):
    >>>     mydummyfunction()  # triggers  dummy_task
    >>> # check_activation_status was automatically called
    """

    def __init__(self, taskname=ALL):
        self.taskname = taskname
        self.calls = []

    def start(self):
        try:
            _mocked_apply_async = _mocked_fn.start()
            _mocked_apply_async.side_effect = _apply_async
        except TypeError:
            pass
        already_mocked = (
            ALL in _mocked_tasks or
            self.taskname in _mocked_tasks or
            (self.taskname == ALL and _mocked_tasks)
        )
        if already_mocked:
            raise AlreadyMockedError()
        _mocked_tasks[self.taskname] = self
        return self

    def stop(self, run_tasks=True):
        if _mocked_tasks.get(self.taskname) != self:
            raise NotStartedError()

        if run_tasks:
            self.runtasks()
        del _mocked_tasks[self.taskname]

    def runtasks(self):
        while self.calls:
            task, args, kwargs = self.calls.pop(0)
            _super_apply_async(task, args=args, kwargs=kwargs)

    def __enter__(self):
        return self.start()

    def __exit__(self, *exc_info):
        self.stop()
        if not _mocked_tasks:
            _mocked_fn.stop()


def _apply_async(selftask, a=None, kw=None, *args, **kwargs):
    tmock = _mocked_tasks.get(ALL) or _mocked_tasks.get(selftask.name)
    if tmock:
        tmock.calls.append((selftask, a, kw))
        return EagerResult(uuid(), None, states.PENDING)

    return _super_apply_async(selftask, *args, **kwargs)


_mocked_fn = mock.patch.object(
    Task, 'apply_async', autospec=True,
)
task_mock = CeleryTaskMock
del CeleryTaskMock



# -*- coding: utf-8 -*-
try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

__version__ = importlib_metadata.version(__name__)
__author__ = """Konrad Rotkiewicz"""
__email__ = 'konrad@ulam.io'

from .celery_mock import task_mock  # noqa



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['celery_mock']

package_data = \
{'': ['*']}

install_requires = \
['celery>=4.0.0']

setup_kwargs = {
    'name': 'celery-mock',
    'version': '1.1.7',
    'description': 'celery-mock allows you to mock celery task and run them when you want',
    'long_description': "===============================\nCelery Task Mocking\n===============================\n\n\n.. image:: https://img.shields.io/pypi/v/celery-mock.svg\n        :target: https://pypi.python.org/pypi/celery-mock\n\n.. image:: https://img.shields.io/travis/ulamlabs/celery-mock.svg\n        :target: https://travis-ci.org/ulamlabs/celery-mock\n\n.. image:: https://codecov.io/gh/ulamlabs/celery-mock/branch/master/graph/badge.svg\n  :target: https://codecov.io/gh/ulamlabs/celery-mock\n\n\ncelery-mock allows you to mock celery task and then run them when you want\n\nRationale\n==========\n\nSometimes a celery task retries itself to wait for some event or model to change.\nThis is hard to test because celery tasks run (and retry) inline in tests.\nNow you can choose when to run your tasks.\n\nSupported versions\n==================\n\n- Python 3 support\n- Celery 3.1.x and 4.0.x support \n\n\nHow to install\n================\n\n    .. code-block:: bash\n    \n        pip install celery-mock\n\n\nHow to use\n==========\n\n    .. code-block:: python\n    \n        from celery_mock import task_mock\n        from django.test import TestCase, Client\n        \n        from myapp import dummyview\n        \n        class UsersTestCase(TestCase):\n         \n            def test_create_user(self):\n                client = Client()\n                client.post('/api/users/', data={'username': 'konrad')  # runs tasks inline\n                \n                with task_mock():\n                    client.post('/api/users/', data={'username': 'konrad')\n                    # no tasks started yet\n                # all tasks ran here\n                \n                with task_mock('myapp.post_user_create_task'):\n                    client.post('/api/users/', data={'username': 'konrad')\n                    # all tasks started execept myapp.post_user_create_task\n                # myapp.post_user_create_task started here\n                \n                # you can use task_mock manually:\n                \n                tmock = task_mock().start()\n                client.post('/api/users/', data={'username': 'konrad')\n                # no tasks started yet\n                tomock.stop()  # all tasks ran here\n",
    'author': 'Konrad Rotkiewicz',
    'author_email': 'konrad@ulam.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ulamlabs/celery-mock/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)
